package ru.mai.k806.sample;

import sun.io.CharToByteEUC_TW;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.sql.SQLException;

public class Quantity implements java.lang.Comparable<Quantity> {
    final BigDecimal m_value;
    final UnitsOfWeightMeasurement m_measure;
    private ConversionService conversionService;

    public void setConversionService(ConversionService conversionService) {
        this.conversionService = conversionService;
    }

    private Quantity(BigDecimal n, UnitsOfWeightMeasurement mes){ m_value=n; m_measure=mes;}
    //преводим к нашим единицам
//    private BigDecimal ConvertToCurrent(Quantity bigDecimal){
//        double factor = bigDecimal.m_measure.ConvertFactorTo(this.m_measure);
//        return bigDecimal.m_value.multiply(BigDecimal.valueOf(factor));
//    }

    public UnitsOfWeightMeasurement Measurement(){ return m_measure; }

    //перевод между единицами
    public Quantity Convert(UnitsOfWeightMeasurement mes){
        //коэффициент преобразования
        double factor = getConversionFactor(mes);
        //умножаем текущие значение на коэффициент
        return new Quantity( m_value.multiply(new BigDecimal(factor)).setScale(m_value.scale(), RoundingMode.HALF_EVEN), mes);
    }

    private double getConversionFactor(UnitsOfWeightMeasurement mes) {
        // return this.m_measure.ConvertFactorTo(mes);
        try {
            return conversionService.convert(m_measure, mes);
        } catch (SQLException e) {
            throw new RuntimeException(e.getMessage(), e);            
        }
    }

    //конструкторы
    public Quantity(int n, UnitsOfWeightMeasurement mes){ m_value=new BigDecimal(n); m_measure =mes;}
    public Quantity(String n, UnitsOfWeightMeasurement mes) {m_value=new BigDecimal(n); m_measure =mes;}

    //статик конструкторы
    public static Quantity ValueOf(int n, UnitsOfWeightMeasurement mes){ return new Quantity(n, mes);}

    //сумма
    public Quantity add(Quantity q){
        //переводим q в нашу систему, складываем значения, делаем новый объект
        return new Quantity( this.m_value.add( q.Convert(this.m_measure).m_value), this.m_measure);
    }

    //вычитание
    public Quantity substruct(Quantity q){
        //переводим q в нашу систему, вычитаем значения, делаем новый объект
        return new Quantity( this.m_value.subtract( q.Convert(this.m_measure).m_value), this.m_measure);
    }

    //умножение
    public Quantity multiply(Quantity q){
        //переводим q в нашу систему, умножаем значения, делаем новый объект
        return new Quantity( this.m_value.multiply( q.Convert(this.m_measure).m_value), this.m_measure);
    }

    //деление
    public Quantity divide(Quantity q){
         //переводим q в нашу систему, делим учитывая MathContext(если делить 1/3 без контекста будет exception),
        // делаем новый объект
        return new Quantity( this.m_value.divide( q.Convert(this.m_measure).m_value, MathContext.DECIMAL128), this.m_measure);
    }

    //деление на n частей
    public Quantity[] Split(int n){

        if(n<=0) return null;

        Quantity[] res = new Quantity[n];

        //порядок текущего значения
        int scale_val = m_value.stripTrailingZeros().scale();
        //количество цифр
        int scale_n = BigDecimal.valueOf(n).precision();

        //часть
        BigDecimal part = this.m_value.divide(BigDecimal.valueOf(n), scale_n+scale_val, RoundingMode.HALF_DOWN);

        //заполняем массив, вычитая часть
        BigDecimal val = m_value;

        for(int i=0 ;i!=n-1; ++i)
        {
           res[i] = new Quantity(part, m_measure);
           val = val.subtract(part);
        }

        //остаток
        res[n-1]=new Quantity( val, m_measure);

        return res;
    }

    //сравнение
    public int compareTo(Quantity q) {
        //переводим в нашу систему и сравниваем по значению
        return this.m_value.compareTo( q.Convert(this.m_measure).m_value);
    }

    public java.lang.String toString(){
        //откидываем завершаюшие нули
        return m_value.stripTrailingZeros().toPlainString() + m_measure.toString();
    }


    public Quantity doubleValue(UnitsOfWeightMeasurement unitsOfWeightMeasurement) {
        return new Quantity(Convert(unitsOfWeightMeasurement).m_value.multiply(BigDecimal.valueOf(2)), unitsOfWeightMeasurement);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Quantity quantity = (Quantity) o;

        if (m_measure != quantity.m_measure) return false;
        if (m_value != null ? !m_value.equals(quantity.m_value) : quantity.m_value != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = m_value != null ? m_value.hashCode() : 0;
        result = 31 * result + (m_measure != null ? m_measure.hashCode() : 0);
        return result;
    }
}

